/* $Id: nws_cache.h,v 1.3 2002/03/12 15:53:35 mquinson Exp $ */

/* nws_cache - generic data container with multiple indexes */

/* this file contains everything client code must see.            */
/*   Datacontainers source code have to load nws_cache_internal.h */

#ifndef __NWS_DC_H
#define __NWS_DC_H
#include "nws_err.h"     /* error codes */
#include "nws_log.h"     /* logging facilities */

/*====[ Key datatype ]=======================================================*/
/*----[ declaration of the possible type ]-----------------------------------*/
#define nwsDcKeyNull_t               0
#define nwsDcKeyShort_t              1
#define nwsDcKeyInt_t                2
#define nwsDcKeyLong_t               3
#define nwsDcKeyUnsignedLong_t       4

#define nwsDcKeyFloat_t              5
#define nwsDcKeyDouble_t             6

#define nwsDcKeyChar_t               7
#define nwsDcKeyString_t             8 /* (const char *) */

typedef struct {
  /* the struct is *copied* around, so i'd better keep it small */
  unsigned short type;
  union {
    short sh;   int i;      long l;     unsigned long ul;
    float f;    double d;   char c;     const char *s;
  } val;
} nwsDcKey_t;  

/*----[ Creation functions ]-------------------------------------------------*/
extern nwsDcKey_t nwsDcKeyShortNew(short s);
extern nwsDcKey_t nwsDcKeyIntNew(int i);
extern nwsDcKey_t nwsDcKeyLongNew(long l);
extern nwsDcKey_t nwsDcKeyUnsignedLongNew(unsigned long ul);
extern nwsDcKey_t nwsDcKeyFloatNew(float f);
extern nwsDcKey_t nwsDcKeyDoubleNew(double d);
extern nwsDcKey_t nwsDcKeyCharNew(char c);
extern nwsDcKey_t nwsDcKeyStringNew(const char *s);
extern nwsDcKey_t nwsDcKeyVoidNew(void *v);

#define nwsDcKeyNULL               ((nwsDcKey_t){nwsDcKeyNull_t,{(double)0}})

/*----[ access macros ]------------------------------------------------------*/
/* Be carfull, no check is done ;)                                           */
/*---------------------------------------------------------------------------*/
#define nwsDcKeyShort(j)        (j.val.sh)
#define nwsDcKeyInt(j)          (j.val.i)
#define nwsDcKeyLong(j)         (j.val.l)
#define nwsDcKeyUnsignedLong(j) (j.val.ul)

#define nwsDcKeyFloat(j)        (j.val.f)
#define nwsDcKeyDouble(j)       (j.val.d)

#define nwsDcKeyChar(j)         (j.val.c)

#define nwsDcKeyVoid(j)         (j.val.v)
#define nwsDcKeyString(j)       (j.val.s)

/*====[ Generic data container types ]=======================================*/
typedef struct nwsDc_ nwsDc_t;
typedef struct nwsDcCell_ nwsDcCell_t;
typedef struct nwsDcCursor_ nwsDcCursor_t;

/*----[ generic interface functions ]----------------------------------------*/
/* calls the right function depending on the type of the datacontainer       */
/*---------------------------------------------------------------------------*/
int nwsDcAdd(nwsDc_t*self,nwsDcKey_t key,void *data,void (*freeData)(void*));
int nwsDcRemove(nwsDc_t*self,nwsDcKey_t key);
int nwsDcRetrieve(nwsDc_t*self,nwsDcKey_t key,/*OUT*/void**where);
int nwsDcCopy(nwsDc_t*dest,nwsDcKey_t destKey,nwsDc_t*src,nwsDcKey_t srcKey);
int nwsDcCommit(nwsDc_t*self);
int nwsDcFree(nwsDc_t *self);
int nwsDcDump(nwsDc_t *self,void (*dumpData)(void*data));

/*----[ for traversal ]------------------------------------------------------*/
int nwsDcCursorNew(nwsDc_t*self,/*OUT*/nwsDcCursor_t **where);
int nwsDcCursorNext(/*IN&OUT*/nwsDcCursor_t *cursor,
		    /*OUT*/nwsDcKey_t *key,void **data);
int nwsDcCursorFree(nwsDcCursor_t *cursor);

/*----[ Specific constructors ]----------------------------------------------*/
/* constructors can't be generic, since they need specific parameters.       */
/* They all have #copyFrom# and #cmpFct# as first parameter:                 */
/*  Any data add or removal in #copyFrom# will be reflected to this          */
/*    datacontainer. If you don't want to copy another dc, just put NULL.    */
/*  #cmpKey# is a fonction to compare the keys. If NULL, a default function  */
/*    is used, it works great if you use numeric or string keys, but can     */
/*    obviously not compare struct passed as (void*)                         */
/*---------------------------------------------------------------------------*/

/*....[ Array type ].........................................................*/
/* simple flat array. Very slow insert/remove, very fast traversal.          */
/*...........................................................................*/
int nwsDcArrayNew(nwsDc_t *owner, int (*cmpKey)(nwsDcKey_t,nwsDcKey_t), 
		  int maxSize,/* OUT */nwsDc_t **where);

/*....[ Linked List type ]...................................................*/
/* simply chained linked list. Very fast insert, quite fast remove and       */
/*   traversal                                                               */
/*...........................................................................*/
int nwsDcListNew(nwsDc_t *copyOf, int (*cmpKey)(nwsDcKey_t,nwsDcKey_t), 
		 /* OUT */nwsDc_t **where);

#endif /* __NWS_DC_H */
