/* $Id: nws_dc_array.c,v 1.3 2002/03/12 15:53:35 mquinson Exp $ */

/* nws_dc_array - data container of type (flat) array */

#include "config.h"
#include "nws_log.h"      /* nws logging facilities */
#include <stdio.h> /* printf for dump. FIXME: Should use logging facility */

#include "nws_cache_internal.h"  /* generic datacontainer */

NWS_LOG_USE(cache);

/*====[ interface implementation ]===========================================*/

/*----[ nwsDcArrayNew ]------------------------------------------------------*/
int nwsDcArrayNew(nwsDc_t *copyOf, int (*cmpKey)(nwsDcKey_t,nwsDcKey_t), 
		  int maxSize,/* OUT */nwsDc_t **where){
  
  if (!(*where=malloc(sizeof(nwsDc_t)))) 
    NWS_OUT_OF_MEM(cache);

  (*where)->magic = nwsDcArrayMagic;
  (*where)->t.a.maxSize = maxSize;
  (*where)->t.a.data=NULL; 
  (*where)->t.a.dataCount=0;

  return nwsDcNew(*where, copyOf,cmpKey);
}

/*----[ nwsDcArrayAdd ]------------------------------------------------------*/
int nwsDcArrayAdd(nwsDc_t*s,nwsDcKey_t key,nwsDcCell_t* cell) {
  if (s->t.a.maxSize <= s->t.a.dataCount) {
    NWS_ERROR(cache)("Array container full. Maxsize=%d",s->t.a.maxSize);
    return NWSERR_FULL;
  }

  s->t.a.dataCount++;
  s->t.a.data=REALLOC(s->t.a.data,sizeof(nwsDcCellArray_t)*s->t.a.dataCount);
  if (!s->t.a.data) NWS_OUT_OF_MEM(cache);
  s->t.a.data[s->t.a.dataCount-1].key=key;
  s->t.a.data[s->t.a.dataCount-1].cell=cell;

  return NWSERR_SUCCES;
}

/*----[ nwsDcArrayRemove ]---------------------------------------------------*/
int nwsDcArrayRemove(nwsDc_t*s,nwsDcCursor_t *cursor) {
  s->t.a.dataCount --;
  if (cursor->t.a != s->t.a.dataCount)
    memcpy(&(s->t.a.data[cursor->t.a]),&(s->t.a.data[cursor->t.a+1]), 
	   sizeof(nwsDcCell_t)*(s->t.a.dataCount - cursor->t.a));

  s->t.a.data=REALLOC(s->t.a.data,sizeof(nwsDcCell_t)*s->t.a.dataCount);
  if (!s->t.a.data && s->t.a.dataCount) NWS_OUT_OF_MEM(cache);

  return NWSERR_SUCCES;
}

/*----[ nwsDcArrayRetrieve ]-------------------------------------------------*/
int nwsDcArrayRetrieve(nwsDc_t*s,nwsDcKey_t key,
		       /*OUT*/nwsDcCursor_t*where) {
  int i;

  for (i=0; 
       i<s->t.a.dataCount && s->cmpKey(key, s->t.a.data[i].key);
       i++);
  if (i==s->t.a.dataCount) {
    return NWSERR_NOTFOUND;
  }
  
  where->magic=nwsDcArrayMagic;
  where->owner=s;
  where->cell=s->t.a.data[i].cell;
  where->t.a=i;
  return NWSERR_SUCCES;
}

/*----[ nwsDcArrayCommit ]---------------------------------------------------*/
int nwsDcArrayCommit(nwsDc_t*self) {
  // This simple array don't cache add. So, nothing to commit.
  return NWSERR_SUCCES;
}

/*----[ nwsDcArrayFree ]-----------------------------------------------------*/
int nwsDcArrayFree(nwsDc_t*s) {
  if (s && s->t.a.data) {
    free(s->t.a.data);
  }
  return NWSERR_SUCCES;
}

int nwsDcArrayCursorInit(nwsDc_t*self,/*OUT*/nwsDcCursor_t *where) {
  where->t.a=-1; /* to have 0 after cursorNext */
  return NWSERR_SUCCES;
}

int nwsDcArrayCursorNext(/*IN&OUT*/nwsDcCursor_t *c,
			 /*OUT*/nwsDcKey_t *key,nwsDcCell_t **data) {

  c->t.a++;
  if (c->t.a < c->owner->t.a.dataCount) {
    *key =c->owner->t.a.data[c->t.a].key;
    *data=c->owner->t.a.data[c->t.a].cell;
    return NWSERR_SUCCES;
  }
  c->t.a--;
  *key=nwsDcKeyNULL;
  *data=NULL;
  return NWSERR_NOTFOUND;
}
int nwsDcArrayCursorFree(nwsDcCursor_t *cursor) {
  return NWSERR_SUCCES;
}

/*----[ nwsDcArrayDump ]-----------------------------------------------------*/
int nwsDcArrayDump(nwsDc_t*s,void (*dumpData)(void*data)) {
  int i;

  printf("Array %p ",s);
  printf("Size=%d,MaxSize=%d,Elements:\n",s->t.a.dataCount,s->t.a.maxSize);
  for (i=0; i<s->t.a.dataCount; i++) {
    printf ("Key=%s;Val=",nwsDcKeyDump(s->t.a.data[i].key));
    if (dumpData) {
      (*dumpData)(s->t.a.data[i].cell->data);
    } else {
      printf("%s\n",(char*)s->t.a.data[i].cell->data);
    }
  }
  printf("End of list of elements\n");

  return NWSERR_SUCCES;
}
