/* $Id: nws_dc_key.c,v 1.2 2002/02/22 04:23:49 mquinson Exp $ */

/* nws_dc_key -- a type that coan hold most of the 8 bytes types  */
/*  (used for keys of data containers)                            */

/* Original idea by Jim Plank (plank@cs.utk.edu) */
/* _heavily_ modified by Martin Quinson for NWS consumtion */

#include <stdio.h>
#include "nws_cache_internal.h" /* the prototype of this module are there */

/*----[ Creation functions ]-------------------------------------------------*/
/* Yes, I'm a lammer, and I can't affect static value to unions.             */
/*   This would allow to write these functions as macro, and inlining them   */
/*   even on platform where inline is not available                          */
/* This works (ie, return is not stuff on the head) because we *copy* the    */
/*   struct back, and don't return an address on them.                       */
/*---------------------------------------------------------------------------*/
nwsDcKey_t nwsDcKeyShortNew(short s) {
  nwsDcKey_t k = {nwsDcKeyShort_t,{0}};
  k.val.sh=s;
  return k;
}
nwsDcKey_t nwsDcKeyIntNew(int i) {
  nwsDcKey_t k = {nwsDcKeyInt_t,{0}};
  k.val.i=i;
  return k;
}
nwsDcKey_t nwsDcKeyLongNew(long l) {
  nwsDcKey_t k = {nwsDcKeyLong_t,{0}};
  k.val.l=l;
  return k;
}
nwsDcKey_t nwsDcKeyUnsignedLongNew(unsigned long ul) {
  nwsDcKey_t k = {nwsDcKeyUnsignedLong_t,{0}};
  k.val.ul=ul;
  return k;
}
nwsDcKey_t nwsDcKeyFloatNew(float f) {
  nwsDcKey_t k = {nwsDcKeyFloat_t,{0}};
  k.val.f=f;
  return k;
}
nwsDcKey_t nwsDcKeyDoubleNew(double d) {
  nwsDcKey_t k = {nwsDcKeyDouble_t,{0}};
  k.val.d=d;
  return k;
}
nwsDcKey_t nwsDcKeyCharNew(char c) {
  nwsDcKey_t k = {nwsDcKeyChar_t,{0}};
  k.val.c=c;
  return k;
}
nwsDcKey_t nwsDcKeyStringNew(const char *s) {
  nwsDcKey_t k = {nwsDcKeyString_t,{0}};
  k.val.s=s;
  return k;
}

/*----[ nwsDcKeyCmp ]--------------------------------------------------------*/
/* The good old strcmp like function.                                        */
/* Numeric values are compared using < and >.                                */
/* String ones are compared using strcmp.                                    */
/* Void* can't be compared (return -2 in any case).                          */
/*                                                                           */
/* Return value:                                                             */
/*    1: a >  b                                                              */
/*    0: a == b                                                              */
/*   -1: a <  b                                                              */
/*   -2: a and b can't be compared. If you're sure of what you're comparing, */
/*       will never happen.                                                  */
/*---------------------------------------------------------------------------*/
#define CMP(a,b) (a<b ? -1 : (a>b ? 1 : 0))

int nwsDcKeyCmp(nwsDcKey_t a,nwsDcKey_t b) {
  /* This function is written in a verbose, but (I hope) efficient manner */
  /*   Ie, I hope that the compiler can produce a compact code with these */
  /*   switch entrelaced                                                  */
  switch(a.type) {

    /****************/
    /* A is numeric */
    /****************/

    /* A is an short */
  case nwsDcKeyShort_t:
    switch(b.type) {
    case nwsDcKeyShort_t:   return CMP(nwsDcKeyShort(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:     return CMP(nwsDcKeyShort(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:    return CMP(nwsDcKeyShort(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:  
      return CMP(nwsDcKeyShort(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   return CMP(nwsDcKeyShort(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  return CMP(nwsDcKeyShort(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    return CMP(nwsDcKeyShort(a),nwsDcKeyChar(b));
    };
    return -2;

    /* A is an integer */
  case nwsDcKeyInt_t:
    switch(b.type) {
    case nwsDcKeyShort_t:   return CMP(nwsDcKeyInt(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:     return CMP(nwsDcKeyInt(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:    return CMP(nwsDcKeyInt(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:  
      return CMP(nwsDcKeyInt(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   return CMP(nwsDcKeyInt(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  return CMP(nwsDcKeyInt(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    return CMP(nwsDcKeyInt(a),nwsDcKeyChar(b));
    };
    return -2;

    /* A is an long */
  case nwsDcKeyLong_t:
    switch(b.type) {
    case nwsDcKeyShort_t:   return CMP(nwsDcKeyLong(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:     return CMP(nwsDcKeyLong(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:    return CMP(nwsDcKeyLong(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:  
      return CMP(nwsDcKeyLong(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   return CMP(nwsDcKeyLong(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  return CMP(nwsDcKeyLong(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    return CMP(nwsDcKeyLong(a),nwsDcKeyChar(b));
    };
    return -2;

    /* A is an unsigned long */
  case nwsDcKeyUnsignedLong_t:
    switch(b.type) {
    case nwsDcKeyShort_t:         
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:           
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:          
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:  
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    
      return CMP(nwsDcKeyUnsignedLong(a),nwsDcKeyChar(b));
    };
    return -2;

    /* A is an float */
  case nwsDcKeyFloat_t:
    switch(b.type) {
    case nwsDcKeyShort_t:   return CMP(nwsDcKeyFloat(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:     return CMP(nwsDcKeyFloat(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:    return CMP(nwsDcKeyFloat(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:
      return CMP(nwsDcKeyFloat(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   return CMP(nwsDcKeyFloat(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  return CMP(nwsDcKeyFloat(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    return CMP(nwsDcKeyFloat(a),nwsDcKeyChar(b));
    };
    return -2;

    /* A is a double */
  case nwsDcKeyDouble_t:
    switch(b.type) {
    case nwsDcKeyShort_t:   return CMP(nwsDcKeyDouble(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:     return CMP(nwsDcKeyDouble(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:    return CMP(nwsDcKeyDouble(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:  
      return CMP(nwsDcKeyDouble(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   return CMP(nwsDcKeyDouble(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  return CMP(nwsDcKeyDouble(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    return CMP(nwsDcKeyDouble(a),nwsDcKeyChar(b));
    };
    return -2;

    /* A is a char */
  case nwsDcKeyChar_t:
    switch(b.type) {
    case nwsDcKeyShort_t:   return CMP(nwsDcKeyChar(a),nwsDcKeyShort(b));
    case nwsDcKeyInt_t:     return CMP(nwsDcKeyChar(a),nwsDcKeyInt(b));
    case nwsDcKeyLong_t:    return CMP(nwsDcKeyChar(a),nwsDcKeyLong(b));
    case nwsDcKeyUnsignedLong_t:  
      return CMP(nwsDcKeyChar(a),nwsDcKeyUnsignedLong(b));
    case nwsDcKeyFloat_t:   return CMP(nwsDcKeyChar(a),nwsDcKeyFloat(b));
    case nwsDcKeyDouble_t:  return CMP(nwsDcKeyChar(a),nwsDcKeyDouble(b));
    case nwsDcKeyChar_t:    return CMP(nwsDcKeyChar(a),nwsDcKeyChar(b));
    };
    return -2;

    /******************/
    /* String element */
    /******************/

  case nwsDcKeyString_t:
    if (b.type==nwsDcKeyString_t) 
      return strcmp(nwsDcKeyString(a),nwsDcKeyString(b));
    return -2;

  }
  
  return -2; /* A must be a (void *) */
}

/*----[ nwsDcKeyDump ]-------------------------------------------------------*/
/* Usefull only when type != void*. Result is overwritten from call to call  */
/*---------------------------------------------------------------------------*/
char *nwsDcKeyDump(nwsDcKey_t k) {
  static char ret[128];
  switch(k.type) {
  case nwsDcKeyNull_t:
    sprintf(ret,"(null)");
    break;
  case nwsDcKeyShort_t:
    sprintf(ret,"%d",k.val.sh);
    break;
  case nwsDcKeyInt_t:
    sprintf(ret,"%d",k.val.i);
    break;
  case nwsDcKeyLong_t:
    sprintf(ret,"%ld",k.val.l);
    break;
  case nwsDcKeyUnsignedLong_t:
    sprintf(ret,"%ld",k.val.ul);
    break;

  case nwsDcKeyFloat_t:
    sprintf(ret,"%f",k.val.f);
    break;
  case nwsDcKeyDouble_t:
    sprintf(ret,"%f",k.val.d);
    break;

  case nwsDcKeyChar_t:
    sprintf(ret,"%c",k.val.c);
    break;
  case nwsDcKeyString_t:
    snprintf(ret,128,"%s",k.val.s);
    break;
  }
  return ret;
}
