/* $Id: nws_dc_list.c,v 1.1 2002/03/12 15:53:35 mquinson Exp $ */

/* nws_dc_list - data container of type (simply) linked list */

#include "config.h"
#include "nws_log.h"      /* nws logging facilities */

#include <stdio.h> /* printf for dump. FIXME: Should use logging facility */

#include "nws_cache_internal.h"  /* generic datacontainer */

NWS_LOG_USE(cache);

/*====[ interface implementation ]===========================================*/

/*----[ nwsDcListNew ]-------------------------------------------------------*/
int nwsDcListNew(nwsDc_t *copyOf, int (*cmpKey)(nwsDcKey_t,nwsDcKey_t), 
		 /* OUT */nwsDc_t **where){
  
  if (!(*where=malloc(sizeof(nwsDc_t)))) 
    NWS_OUT_OF_MEM(cache);

  (*where)->magic = nwsDcListMagic;
  (*where)->t.l.head=NULL; 

  return nwsDcNew(*where, copyOf,cmpKey);
}

/*----[ nwsDcListAdd ]------------------------------------------------------*/
int nwsDcListAdd(nwsDc_t*s,nwsDcKey_t key,nwsDcCell_t* cell) {
  nwsDcCellList_t *c;

  c=malloc(sizeof(nwsDcCellList_t));
  if (!c) NWS_OUT_OF_MEM(cache);

  c->prev=NULL;
  c->next=s->t.l.head;
  c->key=key;
  c->cell=cell;
  s->t.l.head=c;

  return NWSERR_SUCCES;
}

/*----[ nwsDcListRemove ]---------------------------------------------------*/
int nwsDcListRemove(nwsDc_t*s,nwsDcCursor_t *c) {
  
  NWS_VERBOSE(cache)("nwsDcListRemove(%p) cell=%p",c,c?c->cell:NULL);

  /* update head if needed */
  if (c->t.l.this == s->t.l.head && s->t.l.head)
    s->t.l.head = s->t.l.head->next;
  /* update cursor */
  if (c->t.l.this) {
    if (c->t.l.this->prev) 
      c->t.l.this->prev->next = c->t.l.this->next;
    if (c->t.l.this->next)
      c->t.l.this->next->prev = c->t.l.this->prev;
    c->t.l.this = c->t.l.this->prev;
  }

  return NWSERR_SUCCES;
}

/*----[ nwsDcListRetrieve ]-------------------------------------------------*/
int nwsDcListRetrieve(nwsDc_t*self,nwsDcKey_t key,
		       /*OUT*/nwsDcCursor_t*where) {
  
  nwsDcCellList_t *c;
  for (c=self->t.l.head ; 
       c && nwsDcKeyCmp(key,c->key) ; 
       c=c->next);
  if (!c) 
    return NWSERR_NOTFOUND;

  where->magic=nwsDcListMagic;
  where->owner=self;
  where->cell=c->cell;
  where->t.l.this=c;

  return NWSERR_SUCCES;
}

/*----[ nwsDcListCommit ]---------------------------------------------------*/
int nwsDcListCommit(nwsDc_t*self) {
  // This simple array don't cache add. So, nothing to commit.
  return NWSERR_SUCCES;
}

/*----[ nwsDcListFree ]-----------------------------------------------------*/
int nwsDcListFree(nwsDc_t*self) {
  return NWSERR_SUCCES;
}

int nwsDcListCursorInit(nwsDc_t*self,/*OUT*/nwsDcCursor_t *where) {
  NWS_VERBOSE(cache)("nwsDcListCursorInit for %p", self);

  where->t.l.this = NULL; /* to have 0 after cursorNext */
  return NWSERR_SUCCES;
}

int nwsDcListCursorNext(/*IN&OUT*/nwsDcCursor_t *cursor,
			 /*OUT*/nwsDcKey_t *key,nwsDcCell_t **data) {
  int ret;

  NWS_VERBOSE(cache)("nwsDcListCursorNext(%p) Avant : %p %p %p", cursor,
		     cursor->t.l.this ? cursor->t.l.this->prev : NULL,
		     cursor->t.l.this,
		     cursor->t.l.this ? cursor->t.l.this->next : NULL);

  if (!cursor->t.l.this) {
    cursor->t.l.this = cursor->owner->t.l.head;
    ret = cursor->t.l.this ? NWSERR_SUCCES : NWSERR_NOTFOUND;
  } else if (cursor->t.l.this->next)  {
    cursor->t.l.this = cursor->t.l.this->next;
    ret = NWSERR_SUCCES;
  } else {
    ret = NWSERR_NOTFOUND;
  }

  if (ret == NWSERR_SUCCES) {
    *key=cursor->t.l.this->key;
    *data=cursor->t.l.this->cell;
    return NWSERR_SUCCES;
  }
  NWS_VERBOSE(cache)("nwsDcListCursorNext not found");
  *key=nwsDcKeyNULL;
  *data=NULL;
  return NWSERR_NOTFOUND;
}
int nwsDcListCursorFree(nwsDcCursor_t *cursor) {
  return NWSERR_SUCCES;
}

/*----[ nwsDcListDump ]-----------------------------------------------------*/
int nwsDcListDump(nwsDc_t*s,void (*dumpData)(void*data)) {
  nwsDcCellList_t *c;

  printf("List %p\n",s);
  printf("Head=%p; Elements:\n",s->t.l.head);
  for (c= s->t.l.head ; c ; c=c->next) {
    printf ("%p:Key=%s;Cell=%p;Val=",c,nwsDcKeyDump(c->key),c->cell);
    if (c->cell && c->cell->data) {
      if (dumpData) {      
	(*dumpData)(c->cell->data);
      } else {
	printf("%s",(char *)c->cell->data);
      }
    } else {
      printf("(data null)");
    }
    printf("\n");
  }
  printf("End of list of elements\n");

  return NWSERR_SUCCES;
}
