/* $Id: nws_log.h,v 1.3 2002/03/12 15:24:00 mquinson Exp $ */

/* nws_log -- NWS logging subsystem */

#ifndef NWS_LOG_H
#define NWS_LOG_H
#include <stdlib.h>
#include "nws_err.h"
#include <stdarg.h>    /* va_*, vfprintf() */
#include <stdio.h>

/*
 * Logging levels. if you add a level, make sure to update the names in log.c 
 */
typedef enum {
  LogVerbose,  LogDebug,  LogWarn,  LogError, 
  LogLevelCount /*!< Sentinel, not a real level */
} nws_log_level_t;

const char *nwsLogDirectionName(int channel);

/* struct describing a channel */
typedef struct {
  const char* name;                   
  FILE *stream[LogLevelCount];
} nws_log_channel_t;

#define NWS_LOG_CHANNEL_NEW(name) {#name, {NULL, NULL, NULL, NULL}}
#define NWS_LOG_CHANNEL_EMPTY     {NULL,  {NULL, NULL, NULL, NULL}}

/****[ internal stuff, don't mess with it ! ]*********************************/
/* If using GCC, check the args of the logging functions. */
/* Saddly not used since gcc does not allow to set functions attributes to 
   objects being pointers on functions */ 
#ifdef __GNUC__ 
#  define gcc_check_printf __attribute__ ((format (__printf__, 1, 2)))
#else
#  define gcc_check_printf
#endif

int nws_log_position(nws_log_channel_t channel,nws_log_level_t level,
		     const char *file,int line);
void nws_log_printf(const char *fmt, ...) gcc_check_printf;

#define __NWS_LOG_GET(channel,level)  (channel.stream[level])
int __nws_log_parse_option(nws_log_channel_t *channel,const char *optionSet);

#define __NWS_LOG(channel,level) \
     _nws_log_channel=channel,   \
     _nws_log_level=level,       \
     _nws_log_file=__FILE__,     \
     _nws_log_line=__LINE__,     \
    (!__NWS_LOG_GET(channel,level)) ? \
      (void)0 : (nws_log_printf) 
      

/* registration mecanism */
int nws_log_register(const char *name,const char *options,
			     nws_log_channel_t *channel);
#define __NWS_LOG_REGISTER(channel, option) \
  nws_log_register(#channel,option,&__nws_lchan_##channel)
#define __NWS_LOG_DECLARE(channel,options)  \
nws_log_channel_t __nws_lchan_##channel = NWS_LOG_CHANNEL_NEW(channel);

/* backtrace */
void __nws_backtrace(nws_log_channel_t channel);


/****[ exported interface ]***************************************************/
/* Parse the options concerning the logging sub-system */
void nws_log_init(int* argc_addr,const char **argv);

#define NWS_LOG_USE(channel) \
extern nws_log_channel_t __nws_lchan_##channel; \
 /* refs to the globals used to positioning the logging */ \
 extern nws_log_channel_t _nws_log_channel;                \
 extern int _nws_log_level;                                \
 extern const char *_nws_log_file;                         \
 extern int _nws_log_line; 

#define NWS_LOG_DECLARE(channel,options)  NWS_LOG_USE(channel)
#define NWS_LOG_INIT(channel) \
nws_log_register(&__nws_lchan_##channel)

/* use functions */
#define NWS_LOG(channel, level)    __NWS_LOG(__nws_lchan_##channel,level)
#define NWS_LOG_ON(channel, level) __NWS_LOG_GET(__nws_lchan_##channel,level)
#define NWS_LOG_PARSE_OPTION(option )  __nws_log_parse_option(NULL,option)
#define NWS_BACKTRACE(channel,level) \
    if (NWS_LOG_ON(channel,level)) __nws_backtrace(__nws_lchan_##channel)

/*---[ Macros for lazzy programmers ]----------------------------------------*/
#define NWS_VERBOSE(channel)   NWS_LOG(channel,LogVerbose)
#define NWS_DEBUG(channel)     NWS_LOG(channel,LogDebug)
#define NWS_WARN(channel)      NWS_LOG(channel,LogWarn)
#define NWS_ERROR(channel)     NWS_LOG(channel,LogError)

#define NWS_VERBOSE_ON(channel) NWS_LOG_ON(channel,LogVerbose)
#define NWS_DEBUG_ON(channel)   NWS_LOG_ON(channel,LogDebug)
#define NWS_WARN_ON(channel)    NWS_LOG_ON(channel,LogWarn)
#define NWS_ERROR_ON(channel)   NWS_LOG_ON(channel,LogError)

/*----[ common errors ]------------------------------------------------------*/
/* Report on channel that there is not enough memory */
#define NWS_OUT_OF_MEM(channel) \
  do { \
    NWS_ERROR(channel)("Out of memory"); \
    return NWSERR_NOMEM; \
  } while (0)

/* Test the magic (after casting) and do what needed if bad. */
#define NWS_TEST_MAGIC(obj,magicnb) \
  if (obj->magic != magicnb) { \
    NWS_ERROR(cache) \
      ("Damnit! Data have the bad magic number. Expected %d, got %d.",\
        magicnb,obj->magic);\
    return NWSERR_BADMAGIC; \
  } if (0) exit(1)

#endif /* NWS_LOG_H */
