/* $Id: test_cache.c,v 1.3 2002/03/12 15:53:35 mquinson Exp $ */

/* test_cache - Let's crash my code. */

#include <stdio.h>
#include <string.h>
#include "nws_cache.h"

NWS_LOG_USE(cache);
NWS_LOG_DECLARE(test_cache,"*=1");

/* This test code use and char* data. Here are the need functions */
void dumpData(void *elem);
void freeData(void *elem);

void dumpData(void *elem) {
  printf("Data=%s\n",(char*)elem);
}

void freeData(void*elem) {
  if (elem) free(elem);
}

/*****[ The tests ]*******************************************************/
/*fill with some numerical entries, remove them*/
int test_use_num(nwsDc_t *dc);
/*fill with some char&string entries, remove them*/
int test_use_char(nwsDc_t *dc);

/* crash test add+traverse&remove, numerical test */
int test_crash_traverseRemove_num(nwsDc_t*dc,int size,int xp);

int test_traversal(nwsDc_t *dc); /* traverse the dc, and change values */
int traverse_and_remove(nwsDc_t *dc);/* traverse and remove each cell */
int fill_somenum(nwsDc_t *dc); /* fill with some numerical entries */
int fill_n_int(nwsDc_t *dc, int n);/* fill with n integers in [0, 5000[ */
int traverseAndChange(nwsDc_t *dc); /* traverse the dc, and change it */

/* fill with some numerical entries, and remove them*/
int test_use_num(nwsDc_t *dc) {
  return nwsDcAdd(dc,nwsDcKeyShortNew(1),(void*)strdup("short 1"),&freeData) ||
    nwsDcAdd(dc,nwsDcKeyIntNew(2),(void*)strdup("(int) 2"),&freeData) ||
    nwsDcAdd(dc,nwsDcKeyLongNew(3),(void*)strdup("(long) 3"),&freeData) ||
    nwsDcAdd(dc,nwsDcKeyUnsignedLongNew(4),(void*)strdup("(ul) 4"),
	     &freeData) ||
    nwsDcAdd(dc,nwsDcKeyFloatNew(5.5),(void*)strdup("(float) 5.5"),
	     &freeData) ||
    nwsDcAdd(dc,nwsDcKeyDoubleNew(6.5),(void*)strdup("(double) 6.5"),
	     &freeData) ||

    nwsDcRemove(dc,nwsDcKeyShortNew(1)) || 
    nwsDcRemove(dc,nwsDcKeyIntNew(2))   || 
    nwsDcRemove(dc,nwsDcKeyLongNew(3))  ||
    nwsDcRemove(dc,nwsDcKeyUnsignedLongNew(4)) ||
    nwsDcRemove(dc,nwsDcKeyFloatNew(5.5)) ||
    nwsDcRemove(dc,nwsDcKeyDoubleNew(6.5));
}

/*fill with some char&string entries, remove them*/
int test_use_char(nwsDc_t *dc) {
  return nwsDcAdd(dc,nwsDcKeyCharNew('g'),(void*)strdup("(char) g"),
		  &freeData) ||
    nwsDcAdd(dc,nwsDcKeyStringNew("string"),
	     (void*)strdup("(string) string"),&freeData) ||
    nwsDcAdd(dc,nwsDcKeyStringNew("NULL data"),NULL,&freeData) ||

    nwsDcRemove(dc,nwsDcKeyCharNew('g')) ||
    nwsDcRemove(dc,nwsDcKeyStringNew("string")) ||
    nwsDcRemove(dc,nwsDcKeyStringNew("NULL data"));
}

/* crash test add+traverse&remove, numerical test */
int test_crash_traverseRemove_num(nwsDc_t*dc,int size,int xp) {
  int i;

  NWS_DEBUG(test_cache)("Crash test: %d times: add %d elems and remove them. "
		    "(Take time because we strdup data, and have index on it)",
		    xp,size);

  for (i=0;i<xp;i++) {
    printf(".");fflush(stdout);
    if (nwsDcDump(dc,NULL))
      return 1;
    if (fill_n_int(dc,size))
      return 1;
    if (nwsDcDump(dc,NULL))
      return 1;
    if  (traverse_and_remove(dc))
      return 1;
  }
  printf("\n");
  return 0;
}

/*----[ traverse the dc, and change it ]---------------------------------*/
int traverseAndChange(nwsDc_t *dc) {
  nwsDcCursor_t *cursor;
  nwsDcKey_t key;
  void *data;
  int retcode;

  for (retcode=nwsDcCursorNew(dc,&cursor);
       !retcode && (retcode=nwsDcCursorNext(cursor,&key,&data));
       ) {

    if (data) ((char*)data)[0]='!';
  }
  return retcode || nwsDcCursorFree(cursor);
}

int traverse_and_remove(nwsDc_t *dc) {
  nwsDcCursor_t *cursor;
  nwsDcKey_t key;
  void *data;
  int retcode=0;

  while (!retcode) {
    retcode=nwsDcCursorNew(dc,&cursor);
    if (!retcode)
      retcode = nwsDcCursorNext(cursor,&key,&data);
    if (retcode == NWSERR_NOTFOUND) 
      return nwsDcCursorFree(cursor);
    if (!retcode)
      retcode = nwsDcCursorFree(cursor);
    if (!retcode)
      retcode = nwsDcRemove(dc,key);
  }

  if (retcode == NWSERR_NOTFOUND) return NWSERR_SUCCES;
  return retcode;
}

/* fill with n integers between 0 and 5000*/
int fill_n_int(nwsDc_t *dc, int n) {
  int i;
  int retcode=0;

  for (i=0;i<n;i++) {
    retcode = retcode ||
      nwsDcAdd(dc,nwsDcKeyIntNew(/*FIXMErand()*5000*/i),(void*)strdup("AZE"),&freeData);
  }
  return retcode;
}


#define DC_SIZE 10
#define NB_XP 5

int main(int argc,const char **argv) {
  nwsDc_t *dc,*dc2,*dc3;

  nws_log_init(&argc,argv);

  /*
  NWS_DEBUG(test_cache)("Test all types of keys for ARRAY (with ARRAY copy)");
  if (nwsDcArrayNew(NULL,NULL,DC_SIZE,&dc)) exit(1);
  if (nwsDcArrayNew(dc,NULL,DC_SIZE,&dc2)) exit(1);
  test_use_num(dc);
  test_use_char(dc);
  test_crash_traverseRemove_num(dc,DC_SIZE,NB_XP);
  if (nwsDcFree(dc)) exit(1);
  if (nwsDcFree(dc2)) exit(1);
  */

  NWS_DEBUG(test_cache)("XXX Test all types of keys for LIST (with ARRAY copy)");
  if (nwsDcListNew(NULL,NULL,&dc)) exit(1);
  if (nwsDcArrayNew(dc,NULL,DC_SIZE,&dc2)) exit(1);
  test_use_num(dc);
  test_use_char(dc);
  test_crash_traverseRemove_num(dc,DC_SIZE,1);
  if (nwsDcFree(dc)) exit(1);
  if (nwsDcFree(dc2)) exit(1);

  NWS_DEBUG(test_cache)("XXX Test all types of keys for LIST (with LIST copy)");
  if (nwsDcListNew(NULL,NULL,&dc)) exit(1);
  if (nwsDcListNew(dc,NULL,&dc2)) exit(1);
  test_use_num(dc);
  test_use_char(dc);
  test_crash_traverseRemove_num(dc,DC_SIZE,2);
  if (nwsDcFree(dc)) exit(1);
  if (nwsDcFree(dc2)) exit(1);

  NWS_DEBUG(test_cache)
    ("Make a string of 3 copy of indexes, and free them in order");
  if (nwsDcListNew(NULL,NULL,&dc)) exit(1);
  if (nwsDcArrayNew(dc,NULL,DC_SIZE,&dc2)) exit(1);
  if (nwsDcArrayNew(dc2,NULL,DC_SIZE,&dc3)) exit(1);
  if (fill_n_int(dc,DC_SIZE)) exit (1);
  if (nwsDcFree(dc)) exit(1);
  if (nwsDcFree(dc2)) exit(1);
  if (nwsDcFree(dc3)) exit(1);
  NWS_DEBUG(test_cache)("Idem, and free them in reverse order");
  if (nwsDcArrayNew(NULL,NULL,DC_SIZE,&dc)) exit(1);
  if (nwsDcArrayNew(dc,NULL,DC_SIZE,&dc2)) exit(1);
  if (nwsDcArrayNew(dc2,NULL,DC_SIZE,&dc3)) exit(1);
  if (fill_n_int(dc,DC_SIZE)) exit (1);
  if (nwsDcFree(dc3)) exit(1);
  if (nwsDcFree(dc2)) exit(1);
  if (nwsDcFree(dc)) exit(1);
  NWS_DEBUG(test_cache)("Idem, and free them in mixed order");
  if (nwsDcArrayNew(NULL,NULL,DC_SIZE,&dc)) exit(1);
  if (nwsDcArrayNew(dc,NULL,DC_SIZE,&dc2)) exit(1);
  if (nwsDcArrayNew(dc2,NULL,DC_SIZE,&dc3)) exit(1);
  if (fill_n_int(dc,DC_SIZE)) exit (1);
  if (nwsDcFree(dc)) exit(1);
  if (nwsDcFree(dc3)) exit(1);
  if (nwsDcFree(dc2)) exit(1);


  return 0;
}
